/*
	Binary Search Tree
	-- holds channel info for creating channel declaration line

AUTHOR:		L. A. Coon
ORGANIZATION:	Rochester Institute of Technology
EMAIL:		lac@cs.rit.edu

COPYRIGHT:	1994 Lawrence A. Coon
		All rights reserved.

		Permission to copy or use this software without fee is granted
		to non-profit and not-for-profit institutions and organizations
		provided that:

		1) such copying or use is not for commercial advantage, and
		2) no portion of this software is incorporated into any other
                   program or device for commercial advantage, and
		3) this copyright notice remains with the software.

		Permission is also granted to make this software available to
		other non-profit or not-for-profit institutions and
		organizations whose use satisfies (1) through (3) above.
		Any use of this software for commercial advantage, or
		incorporation of any part of this software into any other
		program or device for commercial advantage, or any use
		or copying other than by non-profit or not-for-profit
		institutions or organizations requires written permission of
		the copyright holder.

WARRANTY:	This software is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY.  Neither the author nor distributor
		accepts responsibility to anyone for the consequences of using
		it or for whether it serves any particular purpose or works at
		all.
*/

#include <stdio.h>

#define TRUE 1
#define FALSE 0


char *strdup();

typedef struct tnode
	{
	char *name;
	struct tnode *left, *right;
	} *Treeptr, Treenode;

/*
 *	set up the tree
 */
#define TNULL (Treeptr)'\0'

Treeptr tree = TNULL;

/*
 *	Tree insertion
 */

/* the insertion routine */
bsadd(chan_name)
char chan_name[];
{
Treeptr ptr, previous, new;
int cmp;

	/* find where to put it */
	ptr = tree;
	previous = TNULL;
	while (ptr != TNULL)
	{
		previous = ptr;
		cmp = strcmp(chan_name,ptr->name);
		if (cmp < 0)
			ptr = ptr->left;
		else if (cmp > 0)
			ptr = ptr->right;
		else /* cmp = 0 => duplicate entry -- return error flag */
			return FALSE;
	}

	/* create the new node */
	new = (Treeptr) malloc(sizeof(Treenode));
	if (new == NULL)
	{
		fprintf(stderr,"Malloc failed during channel building\n");
		exit(1);
        }
	new->left = new->right = TNULL;
	new->name = strdup(chan_name);

	/* insert the new node */
	if (previous == TNULL)
		tree = new;
	else if (cmp < 0)
		previous->left = new;
	else
		previous->right = new;
	return TRUE;
}

/*
 *	Tree print
 *
 *	print a comma after every node and a colon at the end
 *	free the nodes as you go, not forgetting the name storage
 */

/*
 * global used to avoid lots of parameter passing during recursion
 */
FILE *file;

/*
 * buffer used to build up the channel declaration line(s).
 */
#define CHAN_STR	"  CHAN OF REAL64"
char chan_str[256] =	CHAN_STR;
char *chan_start =	chan_str + sizeof(CHAN_STR) - 1;
char *chan_end =	chan_str + sizeof(chan_str);
char *chan_ptr;

treedump(fp)
FILE *fp;
{
	file = fp;		/* initialize the global file pointer	*/
	chan_ptr = chan_start;	/* initialize the channel str pointer	*/

	treeprint(tree);	/* print out the channel declaration	*/

	sprintf(--chan_ptr," :\n"); /* back up & overprint the comma	*/
	fputs(chan_str,file);	/* print it to the file	*/
}

treeprint(ptr)
Treeptr ptr;
{
int length;

	if (ptr != TNULL)
	{
		treeprint(ptr->left);
		/* only 256 characters allowed in a declaration line
		   count 'C', space and comma each time
		   count final " :" as one since the comma is counted  */
		length = strlen(ptr->name) + 3;
		if (chan_ptr + length > chan_end )
		{
			sprintf(--chan_ptr," :\n");
			fputs(chan_str,file);
			chan_ptr = chan_start;
		}
		sprintf(chan_ptr," C%s,",ptr->name);
		chan_ptr = chan_ptr + length;
		free(ptr->name);
		free(ptr);
		treeprint(ptr->right);
	}
}
