%{
/*
	yacc program -- for dfg to occam translation

AUTHOR:		L. A. Coon
ORGANIZATION:	Rochester Institute of Technology
EMAIL:		lac@cs.rit.edu

COPYRIGHT:	1994 Lawrence A. Coon
		All rights reserved.

		Permission to copy or use this software without fee is granted
		to non-profit and not-for-profit institutions and organizations
		provided that:

		1) such copying or use is not for commercial advantage, and
		2) no portion of this software is incorporated into any other
                   program or device for commercial advantage, and
		3) this copyright notice remains with the software.

		Permission is also granted to make this software available to
		other non-profit or not-for-profit institutions and
		organizations whose use satisfies (1) through (3) above.
		Any use of this software for commercial advantage, or
		incorporation of any part of this software into any other
		program or device for commercial advantage, or any use
		or copying other than by non-profit or not-for-profit
		institutions or organizations requires written permission of
		the copyright holder.

WARRANTY:	This software is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY.  Neither the author nor distributor
		accepts responsibility to anyone for the consequences of using
		it or for whether it serves any particular purpose or works at
		all.
*/

#include <stdio.h>
#include <strings.h>
#define TRUE 1
#define FALSE 0

/* import the file pointers declared in main */
extern FILE	*fpocc, *fpin, *fpout, *fpbod;

/* some needed globals */
int	lineno = 0;	/* keep count of line number for error reports	*/
char	*message;	/* holds message associated with an IO process	*/
double	value;		/* holds double associated with a constant	*/
int	instr_no;	/* holds instruction number: duplicates not ckd	*/
extern char *opcode_ptr;	/* pointer to opcode in opcode_array -- lex.l	*/

/*
 * stuff for handling input 'variables'
 */

/* flag that triggers printing of input var declaration line --
   FALSE until an input node is read, then TRUE	*/
int is_input =	FALSE;
/* flag that triggers printing of declaration of BOOL required by so.read --
   printing is done on 1st input, then flag is set to TRUE */
int is_bool =	FALSE;

char bool_error[] =	"  BOOL error :\n";
#define INPUT_STR	"  REAL64"
char input_buf[256] =	INPUT_STR;
char *input_start = 	input_buf + sizeof(INPUT_STR) - 1;
char *input_end = 	input_buf + sizeof(input_buf);
char *input_ptr;

%}

%start	pgm

%token	CONST MCONST INPUT MINPUT COPY MERGE XMERGE OUTPUT MESSAGE SWITCH
%token	NOT NEG BIOP
%token	EOS INT REAL STRING OTHER

%%	/* rules section */

pgm	:	/* empty */
	|	pgm stmt EOS
	|	error EOS
		{ printf("illegal instruction on line %d\n", lineno);	 }
	;
stmt	:	inst_no OUTPUT
		{	fprintf(fpout,"          C%d.1 ? outvar\n",instr_no);
			fprintf(fpout,"            SEQ\n");
			fprintf(fpout,
			"              so.write.real64(keyboard,screen,outvar,15,3)\n");
			fprintf(fpout,
			"              so.write.nl(keyboard,screen)\n");
		}
	|	inst_no OUTPUT string
		{	fprintf(fpout,"          C%d.1 ? outvar\n",instr_no);
			fprintf(fpout,"            SEQ\n");
			fprintf(fpout,
			"              so.write.string(keyboard,screen,%s)\n", message);
			fprintf(fpout,
			"              so.write.real64(keyboard,screen,outvar,15,3)\n");
			fprintf(fpout,
			"              so.write.nl(keyboard,screen)\n");
		}
	|	inst_no MESSAGE string
		{	fprintf(fpout,"          C%d.1 ? outvar\n",instr_no);
			fprintf(fpout,
			"            so.write.string.nl(keyboard,screen,%s)\n", message);
		}
	|	inst_no input string node port
		{	save_channel($4, $5);
			/* 1st time decl error variable needed by so.read */
			if ( ! is_bool )
			{
				fputs(bool_error,fpocc);
				is_bool = TRUE;
			}
			/* set/reset flag & input_ptr	*/
			if ( ! is_input )
			{
				is_input = TRUE;
				input_ptr = input_start;
			}
			/* check for input_buf overflow - limit of 99999 vars
			   the 10 is 5 chars for 'var', the space and the
			   comma and 5 for the integer	*/
			if ( input_ptr + 10 > input_end)
			{
				sprintf(--input_ptr," :\n");
				fputs(input_buf,fpocc);
				is_input = FALSE;
			}
			/* create the variable by placing on end of input_buf */
			sprintf(input_ptr," var%d,",instr_no);
			input_ptr = input_buf+strlen(input_buf);

			/* create the input code */
			fprintf(fpin,
			"    so.write.string(keyboard,screen,%s)\n", message);
			fprintf(fpin,
			"    so.read.echo.real64(keyboard,screen,var%d,error)\n", instr_no);
			fprintf(fpin,
			"    so.write.nl(keyboard,screen)\n");
			/* create the constant node procedure call */
			if ($2 == INPUT)
				fprintf(fpbod,
				"      Const(var%d, C%d.%d)\n",instr_no,$4,$5);
			else
				fprintf(fpbod,
				"      Mconst(var%d, C%d.%d)\n",instr_no,$4,$5);
		}
	|	inst_no NEG node port
		{	save_channel($3, $4);
			fprintf(fpbod,
			"      Neg(C%d.1, C%d.%d)\n",instr_no,$3,$4);
		}
	|	inst_no NOT node port
		{	save_channel($3, $4);
			fprintf(fpbod,
			"      Not(C%d.1, C%d.%d)\n",instr_no,$3,$4);
		}
	|	inst_no BIOP node port
		{	save_channel($3, $4);
			fprintf(fpbod, "      %s(C%d.1, C%d.2, C%d.%d)\n",
				opcode_ptr, instr_no, instr_no, $3, $4);
		}
	|	inst_no MERGE node port
		{	save_channel($3, $4);
			fprintf(fpbod,
			"      Merge(C%d.1, C%d.2, C%d.3, C%d.%d)\n",
				instr_no,instr_no,instr_no,$3,$4);
		}
	|	inst_no XMERGE node port
		{	save_channel($3, $4);
			fprintf(fpbod,
			"      Xmerge(C%d.1, C%d.2, C%d.%d)\n",
				instr_no,instr_no,$3,$4);
		}
	|	inst_no CONST value node port
		{	save_channel($4, $5);
			fprintf(fpbod,
			"      Const(%f(REAL64), C%d.%d)\n",value,$4,$5);
		}
	|	inst_no MCONST value node port
		{	save_channel($4, $5);
			fprintf(fpbod,
			"      Mconst(%f(REAL64), C%d.%d)\n",value,$4,$5);
		}
	|	inst_no COPY node port node port
		{	save_channel($3, $4);
			save_channel($5, $6);
			fprintf(fpbod,
			"      Copy(C%d.1, C%d.%d, C%d.%d)\n",
				instr_no,$3,$4,$5,$6);
		}
	|	inst_no SWITCH node port node port
		{	save_channel($3, $4);
			save_channel($5, $6);
			fprintf(fpbod,
			"      Switch(C%d.1, C%d.2, C%d.%d, C%d.%d)\n",
				instr_no,instr_no,$3,$4,$5,$6);
		}
	|	OTHER
	;
inst_no	:	INT
		{ instr_no = yylval;	}
	;
node	:	INT
	;
port	:	INT
	;
value	:	REAL
	;
string	:	STRING
	;
input	:	INPUT
		{	$$ = INPUT;	}
	|	MINPUT
		{	$$ = MINPUT;	}
	;
