%{
/*
	lex translator -- data flow graph code -> occam 2

AUTHOR:		L. A. Coon
ORGANIZATION:	Rochester Institute of Technology
EMAIL:		lac@cs.rit.edu

COPYRIGHT:	1994 Lawrence A. Coon
		All rights reserved.

		Permission to copy or use this software without fee is granted
		to non-profit and not-for-profit institutions and organizations
		provided that:

		1) such copying or use is not for commercial advantage, and
		2) no portion of this software is incorporated into any other
                   program or device for commercial advantage, and
		3) this copyright notice remains with the software.

		Permission is also granted to make this software available to
		other non-profit or not-for-profit institutions and
		organizations whose use satisfies (1) through (3) above.
		Any use of this software for commercial advantage, or
		incorporation of any part of this software into any other
		program or device for commercial advantage, or any use
		or copying other than by non-profit or not-for-profit
		institutions or organizations requires written permission of
		the copyright holder.

WARRANTY:	This software is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY.  Neither the author nor distributor
		accepts responsibility to anyone for the consequences of using
		it or for whether it serves any particular purpose or works at
		all.
*/

#include <stdio.h>
#include "y.tab.h"

extern int lineno;
extern int yylval;
extern char *message;	/* holds message associated with an IO process  */
extern double value;	/* holds double associated with a constant      */
char *strdup();
double atof();
int atoi();

/* handle binary ops by returning BIOP, and setting a pointer to the
   actual opcode -- to keep yacc tables small				*/
char *opcode_ptr;

%}
string	\"[ -~]+\"
digits	([0-9]+)
pt	"."
sign	[+-]?
exp	([eE]{sign}{digits})
other	[a-zA-Z]+
%%
Const	return CONST;
Mconst	return MCONST;
Input	return INPUT;
Minput	return MINPUT;
Copy	return COPY;
Merge	return MERGE;
Xmerge	return XMERGE;
Output	return OUTPUT;
Message	return MESSAGE;
Switch	return SWITCH;
Not	return NOT;
Neg	return NEG;
Fgate	{
	opcode_ptr = "Fgate";
	return BIOP;
	}
Tgate	{
	opcode_ptr = "Tgate";
	return BIOP;
	}
And	{
	opcode_ptr = "And";
	return BIOP;
	}
Or	{
	opcode_ptr = "Or";
	return BIOP;
	}
Eq	{
	opcode_ptr = "Eq";
	return BIOP;
	}
Ge	{
	opcode_ptr = "Ge";
	return BIOP;
	}
Le	{
	opcode_ptr = "Le";
	return BIOP;
	}
Lt	{
	opcode_ptr = "Lt";
	return BIOP;
	}
Gt	{
	opcode_ptr = "Gt";
	return BIOP;
	}
Ne	{
	opcode_ptr = "Ne";
	return BIOP;
	}
Add	{
	opcode_ptr = "Add";
	return BIOP;
	}
Sub	{
	opcode_ptr = "Sub";
	return BIOP;
	}
Mul	{
	opcode_ptr = "Mul";
	return BIOP;
	}
Div	{
	opcode_ptr = "Div";
	return BIOP;
	}
Mod	{
	opcode_ptr = "Mod";
	return BIOP;
	}
{digits}	{
	yylval = atoi(yytext);
	return INT;
	}
{sign}{digits}{pt}{digits}{exp}? { /* needs digits on both sides of the dot */
	value = atof(yytext);
	return REAL;
	}
{string}	{
	if ((message = strdup(yytext)) == NULL)
	{
		fprintf(stderr, "strdup failed\n");
		exit(1);
	}
	return STRING; /* character set dependent */
	}
[ \t]+	;			/* eat white space */; 
^"--".*"\n"	{
		lineno++;	/* ignore full line comments */
	}
"--".*"\n"	{
		lineno++;
		return EOS;	/* ignore comments in statements */
	}
^"\n"   {
		lineno++;	/* ignore blank lines */
	}
"\n"	{
		lineno++;
		return EOS;
	}
{other}	{
	return OTHER;
	}
.	{
	printf("ERROR, illegal character %d\n",yytext[0]);
	exit(1);
	}
%%
FILE *fopen();

FILE *lexopen(s)
char *s;
{
	return yyin = fopen(s,"r");
}

lexrewind()
{
	rewind(yyin);
}
