/*
 	translator -- data flow graph code -> occam 2
 
AUTHOR:		L. A. Coon
ORGANIZATION:	Rochester Institute of Technology
EMAIL:		lac@cs.rit.edu

COPYRIGHT:	1994 Lawrence A. Coon
		All rights reserved.

		Permission to copy or use this software without fee is granted
		to non-profit and not-for-profit institutions and organizations
		provided that:

		1) such copying or use is not for commercial advantage, and
		2) no portion of this software is incorporated into any other
		   program or device for commercial advantage, and
		3) this copyright notice remains with the software.

		Permission is also granted to make this software available to
		other non-profit or not-for-profit institutions and
		organizations whose use satisfies (1) through (3) above.
		Any use of this software for commercial advantage, or
		incorporation of any part of this software into any other
		program or device for commercial advantage, or any use
		or copying other than by non-profit or not-for-profit
		institutions or organizations requires written permission of
		the copyright holder.

WARRANTY:       This software is distributed in the hope that it will be useful,
                but WITHOUT ANY WARRANTY.  Neither the author nor distributor
                accepts responsibility to anyone for the consequences of using
                it or for whether it serves any particular purpose or works at
    	    	all.


USAGE:	gto file.g 

NOTE:	Occam code generated is for the Inmos toolset
 */
#include <stdio.h>
#include "y.tab.h"

#define TRUE 1
#define FALSE 0

FILE *fopen();

/* file pointers for the various parts generated that are cat'd to form pgm */
FILE *fpocc;	/* the whole occam file */
FILE *fpin;	/* the input code	*/
FILE *fpout;	/* the output code	*/
FILE *fpbod;	/* the body containing the procedure calls */

/* variable declaration string stuff -- defined in gram.y		*/
extern int is_input;		/* TRUE if input is done		*/
extern char *input_ptr;		/* ptr to end of input_buf str		*/
extern char *input_start;	/* ptr to start posn in input_buf	*/
extern char input_buf[];	/* variable declarationstring		*/

/*
 * MAIN DRIVER PROGRAM
 */

main(argc, argv)
int argc;
char *argv[];
{
int instr_no, opcode, node, inport, outport, token;
int basename_length;
char c, *name, *filename, *malloc(), *strdup();

/*
 * OPEN THE FILES -- handle typical errors
 */

if (argc == 2)
{
	name = argv[1];
}
else
{
	fputs("Illegal number of arguments\n",stderr);
	fputs("Usage: gto file.g\n",stderr);
	exit (1);
}

/*
 * open the g code file -- do some error checking
 */

/* check for wrong extension */
if ( (name[strlen(name)-1] != 'g') ||
	(name[strlen(name)-2] != '.') )
{
	fprintf(stderr,"Wrong extension, %s should end in '.g'\n", name);
	exit(1);
}

/* check for open failure */
if (lexopen(name) == NULL)
{
	perror(name);
	exit(1);
}

/*
 * CREATE THE OCCAM CODE FILE AND TEMPORARY SUPPORT FILES
 *
 *	create the file name string with room for 3 character extension
 *	the current extension is 1 char, we need room for 3 chars
 *	so add 3 to length: one for null and two for extra chars
 */

if ((filename = malloc(strlen(name) + 3)) == NULL)
{
	perror("Malloc failure creating Occam file name space");
	exit(1);
}

/* copy the file base name */
strcpy(filename,name);

/* kill the 'g' extension but leave the dot */
basename_length = strlen(name)-1;
filename[basename_length] = NULL;

/* open the occam file */
strcat(filename,"occ");

if ((fpocc = fopen(filename,"w")) == NULL)
{
	perror("Occam code file");
        exit(1);
}

/*
 * open the temporary files and unlink them so they go away on exit
 */

/* open the input temp file */
filename[basename_length] = NULL;
strcat(filename,"in");

if ((fpin = fopen(filename,"w+")) == NULL)
{
	perror("Input temp file");
        exit(1);
}
unlink(filename);

/* open the output temp file */
filename[basename_length] = NULL;
strcat(filename,"out");

if ((fpout = fopen(filename,"w+")) == NULL)
{
	perror("Output temp file");
        exit(1);
}
unlink(filename);

/* open the body temp file */
filename[basename_length] = NULL;
strcat(filename,"bod");

if ((fpbod = fopen(filename,"w+")) == NULL)
{
	perror("Body temp file");
        exit(1);
}
unlink(filename);

/*
 * GENERATE THE CODE
 */

/*
 * Do some preliminary stuff to files before calling yacc to fill them in 
 */

/* Start by printing the initial housekeeping stuff to the occam code file */
fputs("#INCLUDE \"hostio.inc\"\n",fpocc);
fputs("PROC main(CHAN OF SP keyboard, screen, [] INT memory)\n",fpocc);
fputs("  #USE \"convert.lib\"\n",fpocc);
fputs("  #USE \"hostio.lib\"\n",fpocc);
fputs("  #USE \"DF.lib\"\n",fpocc);

/* Print the PAR to filename.bod before calling yacc */
fputs("    PAR\n",fpbod);

/* Do the SEQ and ALTs needed in filename.out before calling yacc */
fputs("      REAL64 outvar :\n",fpout);
fputs("      WHILE TRUE\n",fpout);
fputs("        ALT\n",fpout);

/*
 * CALL YACC
 *
 *	it writes:
 *		the channel line to filename.occ
 *		input code to filename.in
 *		node procedure calls to filename.bod
 *		output code to filename.out
 *	as it does one pass over filename.g
 */
yyparse();

/* if there're input variable decls that haven't been written out -- do it */
if ( is_input )
{
	/* overwrite last comma with semicolon and print to file */
	sprintf(--input_ptr," :\n");
	fputs(input_buf,fpocc);
}

/*
 * Print out the channel declaration line
 */
treedump(fpocc);

/*
 * Print out the initial SEQ and keyboard assignment
 */
fputs("  SEQ\n",fpocc);

/*
 * Print the closing semi at the end of the out file, as it goes on last
 */
fprintf(fpout,":\n");

/*
 * put all the files together
 */

rewind(fpin);
while ((c = getc(fpin)) != EOF)
	putc(c,fpocc);
rewind(fpbod);
while ((c = getc(fpbod)) != EOF)
	putc(c,fpocc);
rewind(fpout);
while ((c = getc(fpout)) != EOF)
	putc(c,fpocc);
}
